/* $NetBSD$ */

/*-
 * Copyright (c)2006, 2012 Takehiko NOZAKI,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef _TR24731_CONSTRAINT_H_
#define _TR24731_CONSTRAINT_H_

/*
 * arg shall not be a null pointer.
 */
#define assert_not_null(arg) \
if (arg == NULL) \
	fail(CONSTRAINT_NOT_NULL, __STRING(arg))
/*
 * arg shall not equal zero.
 */
#if ~RSIZE_MAX < 0 /* XXX: FIXME */
#define assert_not_zero(arg) \
if (arg < 1) \
	fail(CONSTRAINT_NOT_ZERO, __STRING(arg))
#else
#define assert_not_zero(arg)  \
if (arg == 0) \
	fail(CONSTRAINT_NOT_ZERO, __STRING(arg))
#endif

/*
 * arg shall not be greater than RSIZE_MAX.
 */
#if RSIZE_MAX < ~0 /* XXX: FIXME */
#define assert_rsize_max(arg) \
if (arg > RSIZE_MAX) \
	fail(CONSTRAINT_RSIZE_MAX, __STRING(arg))
#else
#define assert_rsize_max(arg)	/**/
#endif

/*
 * Copying shall not take place between arg1 and arg2 that overlap.
 */
#define assert_not_overlap(arg1, arg2, type, size) \
if ((rsize_t)((const type *)arg2 - (type *)arg1) < size) \
	fail(CONSTRAINT_OVERLAP, \
	    __STRING(arg1), __STRING(arg2))

/*
 * arg1 shall be greater than arg2.
 */
#define assert_gt(arg1, arg2) \
if (arg1 <= arg2) \
	fail(CONSTRAINT_GT, \
	    __STRING(arg1), __STRING(arg2))

/*
 * arg1 shall not be greater than arg2.
 */
#define assert_not_gt(arg1, arg2) \
if (arg1 > arg2) \
	fail(CONSTRAINT_NOT_GT, \
	    __STRING(arg1), __STRING(arg2))

/*
 * arg1 shall be less than arg2.
 */
#define assert_lt(arg1, arg2) \
if (arg1 >= arg2) \
	fail(CONSTRAINT_LT, \
	    __STRING(arg1), __STRING(arg2))

/*
 * arg1 shall not be less than arg2.
 */
#define assert_not_lt(arg1, arg2) \
if (arg1 < arg2) \
	fail(CONSTRAINT_NOT_LT, \
	    __STRING(arg1), __STRING(arg2))

#endif /* _TR24731_CONSTRAINT_H_ */
