/* $NetBSD$ */

/*-
 * Copyright (c)2006, 2012 Takehiko NOZAKI,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <sys/cdefs.h>
#if defined(LIBC_SCCS) && !defined(lint)
#ifdef __NetBSD__
__RCSID("$NetBSD$");
#endif
#endif /* LIBC_SCCS and not lint */

#include <errno.h>
#include <stddef.h>
#include <stdio.h>
#include <stdint.h>

#include "tr24731_local.h"
#define fail(err, ...) \
do { \
	if (newstreamptr != NULL) \
		*newstreamptr = NULL; \
	return __runtime_error(err, __func__, __VA_ARGS__); \
	/* NOTREACHED */ \
} while (/*CONSTCOND*/0)
#include "tr24731_constraint.h"

errno_t
freopen_s(FILE * __restrict * __restrict newstreamptr,
	const char * __restrict filename, const char * __restrict mode,
	FILE * __restrict stream)
{
	FILE *fp;
	int retval, sverrno;

	assert_not_null(newstreamptr);
	assert_not_null(mode);
	assert_not_null(stream);

	retval = 0;
	sverrno = errno;
	fp = freopen(filename, mode, stream);
	if (fp == NULL) {
		retval = errno;
		errno = sverrno;
	}
	*newstreamptr = fp;

	return (errno_t)retval;
}
