/* $NetBSD$ */

/*-
 * Copyright (c)2006, 2012 Takehiko NOZAKI,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <sys/cdefs.h>
#if defined(LIBC_SCCS) && !defined(lint)
#ifdef __NetBSD__
__RCSID("$NetBSD$");
#endif
#endif /* LIBC_SCCS and not lint */

#include "reentrant.h"
#include <assert.h>
#ifndef _DIAGASSERT
#define _DIAGASSERT(exp)	assert(exp)
#endif
#include <errno.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>

#include "tr24731_local.h"

static const constraint_err __constraint_errlist[] = {
{ 0,      NULL, }, /* dummy */
{ EINVAL, "%s, %s shall not be a null pointer.\n" },
{ ERANGE, "%s, %s shall not qual zero.\n" },
{ ERANGE, "%s, %s shall not be greater than RSIZE_MAX.\n" },
{ EINVAL, "%s, Copying shall not take place between %s and %s that overlap.\n" },
{ ERANGE, "%s, %s shall be greater than %s.\n" },
{ ERANGE, "%s, %s shall not be greater than %s.\n" },
{ ERANGE, "%s, %s shall be less than %s.\n" },
{ ERANGE, "%s, %s shall not be less than %s.\n" },
{ EINVAL, "%s, Neither a new-line character, nor end-of-file "
          "found within reading %s characters from stdin.\n" },
};

const constraint_err *constraint_errlist = __constraint_errlist;
size_t nconstraint_errlist =
	sizeof(__constraint_errlist) / sizeof(constraint_err);

errno_t
__runtime_error(int msgid, ...)
{
	extern constraint_handler_t __constraint_handler;
#ifdef _REENTRANT
	extern rwlock_t __constraint_handler_lock;
#endif

	const constraint_err *p;
	va_list ap;
	char msg[128 /* XXX: FIXME */];
	size_t n;

	_DIAGASSERT(msgid > 0 && msgid <= nconstraint_errlist);

	p = &constraint_errlist[msgid];
	rwlock_rdlock(&__constraint_handler_lock);
	if (__constraint_handler != NULL) {
		va_start(ap, msgid);
		vsnprintf(msg, sizeof(msg), p->fmt, ap);
		va_end(ap);
		(*__constraint_handler)((const char *)&msg[0], NULL, p->errnum);
	}
	rwlock_unlock(&__constraint_handler_lock);

	return (errno_t)p->errnum;
}
