/* $NetBSD$ */

/*-
 * Copyright (c)2006, 2012 Takehiko NOZAKI,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/*
 * Copyright (c) 1990, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Original version ID:
 *   @(#)bsearch.c	8.1 (Berkeley) 6/4/93
 *   NetBSD: bsearch.c,v 1.12 2003/08/07 16:43:39 agc Exp $
 */

#include <sys/cdefs.h>
#if defined(LIBC_SCCS) && !defined(lint)
#ifdef __NetBSD__
__RCSID("$NetBSD$");
#endif
#endif /* LIBC_SCCS and not lint */

#ifndef __UNCONST
#define __UNCONST(var)	((void *)(unsigned long)(const void *)(var))
#endif

#include <errno.h>
#include <stddef.h>
#include <stdint.h>
#include <stdlib.h>

#include "tr24731_local.h"
#define fail(err, ...) \
do { \
	(void)__runtime_error(err, __func__, __VA_ARGS__); \
	return NULL; \
	/*NOTREACHED*/ \
} while (/*CONSTCOND*/0)
#include "tr24731_constraint.h"

void *
bsearch_s(const void *key, const void *base, rsize_t nmemb, rsize_t size,
	int (*compar)(const void *, const void *, void *), void *context)
{
	rsize_t lim;
	int cmp;
	const void *p;

	assert_rsize_max(nmemb);
	assert_rsize_max(size);
#if defined(COMPAT_WHIDBEY) || defined(COMPAT_OPENWATCOM)
	assert_not_null(key);
#endif
#ifdef COMPAT_WHIDBEY
	if (nmemb > 0)
		assert_not_null(base);
	assert_not_null(compar);
#else
	if (nmemb > 0) {
		assert_not_null(base);
		assert_not_null(compar);
	}
#endif

	for (lim = nmemb; lim != 0; lim >>= 1) {
		p = (const void *)((const char *)base + ((lim >> 1) * size));
		cmp = (*compar)(key, p, context);
		if (cmp == 0)
			return __UNCONST(p);
		if (cmp > 0) {	/* key > p: move right */
			base = (const char *)p + size;
			--lim;
		}		/* else move left */
	}

	return NULL;
}
