/* $NetBSD$ */

/*-
 * Copyright (c)2006, 2012 Takehiko NOZAKI,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/*
 * Copyright (c) 1987, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Original version ID:
 *     @(#)getenv.c	8.1 (Berkeley) 6/4/93
 *     NetBSD: getenv.c,v 1.35 2010/11/14 22:04:36 tron Exp
 */

#include <sys/cdefs.h>
#if defined(LIBC_SCCS) && !defined(lint)
#ifdef __NetBSD__
__RCSID("$NetBSD$");
#endif
#endif /* LIBC_SCCS and not lint */

#include <assert.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>

#include "env.h"
#include "reentrant.h"
#include "local.h"

#include "tr24731_local.h"
#ifndef COMPAT_WHIDBEY
#define set_needed(value) if (needed != NULL) *needed = value
#else
#define set_needed(value) *needed = value
#endif
#define fail(err, ...) \
do { \
	set_needed(0); \
	if (writable != 0) \
		*value = '\0'; \
	return __runtime_error(err, __func__, __VA_ARGS__); \
	/*NOTREACHED*/ \
} while (/*CONSTCOND*/0)
#include "tr24731_constraint.h"

errno_t
getenv_s(size_t * __restrict needed, char * __restrict value,
	rsize_t maxsize, const char * __restrict name)
{
	int writable;
	errno_t errnum;
	rsize_t l_name, l_value;
	char *result;
 
	writable = 0;
	assert_rsize_max(maxsize);
	if (maxsize > 0) {
		assert_not_null(value);
		writable = 1;
	}
#ifdef COMPAT_WHIDBEY
	assert_not_null(needed);
#endif
	assert_not_null(name);

	errnum = (errno_t)EINVAL;
	l_name = (rsize_t)__envvarnamelen(name, false);
	l_value = 0;
	if (l_name > 0) {
		if (__readlockenv()) {
			result = __findenvvar(name, l_name);
			if (result != NULL) {
				l_value = (rsize_t)strlen(result) + 1;
				if (maxsize >= l_value) {
					if (writable != 0)
						memcpy(value, result, l_value);
#ifdef COMPAT_OPENWATCOM
					--l_value;
#endif
					errnum = (errno_t)0;
					writable = 0;
				}
			}
			(void)__unlockenv();
		}
	}
	set_needed(l_value);
	if (writable != 0)
		*value = '\0';
	return errnum;
}
