/* $NetBSD$ */

/*-
 * Copyright (c)2006, 2012 Takehiko NOZAKI,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <sys/cdefs.h>
#if defined(LIBC_SCCS) && !defined(lint)
#ifdef __NetBSD__
__RCSID("$NetBSD$");
#endif
#endif /* LIBC_SCCS and not lint */

#include <errno.h>
#include <stddef.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "tr24731_local.h"
#define fail(err, ...) \
do { \
	return __runtime_error(err, __func__, __VA_ARGS__); \
	/*NOTREACHED*/ \
} while (/*CONSTCOND*/0)
#include "tr24731_constraint.h"

errno_t
strerror_s(char *s, rsize_t maxsize, errno_t errnum)
{
	int idx;
	const char *m;
	size_t n;

	assert_not_null(s);
	assert_rsize_max(maxsize);
	assert_not_zero(maxsize);

	idx = (int)errnum;
	errnum = E2BIG;
	if (idx >= 0 && idx < sys_nerr) {
		/* TODO: nls support */
		m = sys_errlist[idx];
		n = strlen(m) + 1;
		if (n <= maxsize) {
			strlcpy(s, m, n);
			errnum = (errno_t)0;
		} else if (maxsize < 4) {
			strlcpy(s, m, maxsize); /* ??? */
		} else {
			/* XXX: FIXME
			 * TR24731 considered neither multibyte
			 * nor stateful encoding...
			 */
			if ((n = maxsize - 4) > 0) {
				memcpy(s, m, n);
				s += n;
			}
			memcpy(s, "...", 4);
		}
	} else {
		/* XXX: C99 snprintf */
		n = snprintf(NULL, 0, UPREFIX, idx) + 1;
		if (n <= maxsize) {
			snprintf(s, n, UPREFIX, idx);
			errnum = (errno_t)0;
		} else if (maxsize < 4) {
			snprintf(s, maxsize, UPREFIX, idx); /* ??? */
		} else {
			/* XXX: FIXME
			 * TR24731 considered neither multibyte
			 * nor stateful encoding...
			 */
			n = maxsize - 4;
			if ((n = maxsize - 4) > 0) {
				snprintf(s, n + 1, UPREFIX, idx);
				s += n;
			}
			memcpy(s, "...", 4);
		}
	}
	return errnum;
}
