/* $NetBSD$ */

/*-
 * Copyright (c)2006, 2012 Takehiko NOZAKI,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef _STRTOK_S_TEMPLATE_H_
#define	_STRTOK_S_TEMPLATE_H_

#include "tr24731_local.h"
#define fail(err, ...) \
do { \
	(void)__runtime_error(err, __func__, __VA_ARGS__); \
	return NULL; \
	/*NOTREACHED*/ \
} while (/*CONSTCOND*/0)
#include "tr24731_constraint.h"

_CHAR_TYPE *
_FUNCNAME(_CHAR_TYPE * __restrict s1, rsize_t * __restrict s1max,
	const _CHAR_TYPE * __restrict s2, _CHAR_TYPE ** __restrict ptr)
{
	_CHAR_TYPE *result;
	const _CHAR_TYPE *delim;

	assert_not_null(ptr);
	if (s1 == NULL) {
		assert_not_null(*ptr);
		s1 = *ptr;
	}
	assert_not_null(s1max);
	assert_rsize_max(*s1max);
	assert_not_null(s2);

	result = *ptr = NULL;
restart:
	if (*s1max > 0 && *s1 != _L('\0')) {
		for (delim = s2; *delim != _L('\0'); ++delim) {
			if (*s1 == *delim) {
				--(*s1max), ++s1;
				goto restart;
			}
		}
		result = s1;
		do {
			for (delim = s2; *delim != 0; ++delim) {
				if (*s1 == *delim) {
					*s1 = _L('\0');
					*ptr = s1 + 1;
					goto done;
				}
			}
			--(*s1max), ++s1;
		} while (*s1max > 0 && *s1 != _L('\0'));
	}
done:
	return result;
}
#endif /*_STRTOK_S_TEMPLATE_H_*/
