/******************************************************************************

 Copyright (C) 2006 Frank Beythien, Michal Necasek

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 IN THE SOFTWARE.

******************************************************************************/


#define __STDC_WANT_LIB_EXT1__  1

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include "safetest.h"

#if 1
#include <tr24731.h>
#endif


int     NumViolations;                  /* runtime-constraint violation counter */


/* Runtime-constraint handler for tests; doesn't abort program. */
void my_constraint_handler( const char *msg, void *ptr, errno_t error )
{
#ifdef PRINT_VIOLATIONS
    fprintf( stdout, "Runtime-constraint in %s", msg );
#endif
    ++NumViolations;
}


void test_getenv_s( void )
/************************/
{
    char    buf[128];
    size_t  len = 0;
    int     violations = NumViolations;

    VERIFY( putenv( "NAME=VALUE" ) == 0 );

    /* Test the "good" case */
    VERIFY( getenv_s( NULL, buf, sizeof( buf ), "NAME" ) == 0 );
    VERIFY( getenv_s( &len, buf, sizeof( buf ), "NAME" ) == 0 );
#if 0
    VERIFY( len == strlen( "VALUE" ) );
#else
    VERIFY( len == strlen( "VALUE" ) + 1 );
#endif
    VERIFY( strcmp( buf, "VALUE" ) == 0 );
    VERIFY( NumViolations == violations );

    /* Test various failing cases */
    VERIFY( getenv_s( &len, buf, sizeof( buf ), "R$#X8ZY" ) != 0 );
    VERIFY( getenv_s( NULL, buf, sizeof( buf ), "R$#X8ZY" ) != 0 );
    VERIFY( getenv_s( NULL, NULL, 0, "R$#X8ZY" ) != 0 );
    VERIFY( len == 0 );
    VERIFY( buf[0] == '\0' );

    VERIFY( getenv_s( &len, buf, 4, "NAME" ) != 0 );
#if 0
    VERIFY( len == strlen( "VALUE" ) );
#else
    VERIFY( len == strlen( "VALUE" ) + 1 );
#endif
    VERIFY( NumViolations == violations );

    VERIFY( getenv_s( &len, NULL, 0, "NAME" ) != 0 );
#if 0
    VERIFY( len == strlen( "VALUE" ) );
#else
    VERIFY( len == strlen( "VALUE" ) + 1);
#endif
    VERIFY( NumViolations == violations );

    /* Test runtime-constraint violations */
#if 0
    VERIFY( getenv_s( &len, NULL, sizeof( buf ), "NAME" ) != 0 );
    VERIFY( NumViolations == ++violations );

    VERIFY( getenv_s( &len, buf, sizeof( buf ), NULL ) != 0 );
    VERIFY( NumViolations == ++violations );

#if RSIZE_MAX != SIZE_MAX
    VERIFY( getenv_s( &len, buf, ~0, "NAME" ) != 0 );
    VERIFY( NumViolations == ++violations );
#endif
#endif
}

int main( int argc, char **argv )
/*******************************/
{
    set_constraint_handler_s( my_constraint_handler );

    test_getenv_s();

    printf( "Tests completed (%s).\n", __FILE__ );

    return( 0 );
}
